@@ -2,6 +2,8 @@
    Please see the LICENSE file for copyright and distribution information */
 
 #include "rp_call_tree.h"
+#include "rp_call_trees.h"
+#include "rp_thread.h"
 
 VALUE cRpCallTree;
 
@@ -9,6 +11,7 @@ VALUE cRpCallTree;
 prof_call_tree_t* prof_call_tree_create(prof_method_t* method, prof_call_tree_t* parent, VALUE source_file, int source_line)
 {
     prof_call_tree_t* result = ALLOC(prof_call_tree_t);
+    result->owner = OWNER_C;
     result->method = method;
     result->parent = parent;
     result->object = Qnil;
@@ -23,34 +26,12 @@ prof_call_tree_t* prof_call_tree_create(prof_method_t* method, prof_call_tree_t*
 
 prof_call_tree_t* prof_call_tree_copy(prof_call_tree_t* other)
 {
-    prof_call_tree_t* result = ALLOC(prof_call_tree_t);
-    result->children = rb_st_init_numtable();
-    result->object = Qnil;
-    result->visits = 0;
-
-    result->method = other->method;
-    result->parent = other->parent;
-    result->source_line = other->source_line;
-    result->source_file = other->source_file;
-
-    result->measurement = prof_measurement_create();
-    result->measurement->called = other->measurement->called;
-    result->measurement->total_time = other->measurement->total_time;
-    result->measurement->self_time = other->measurement->self_time;
-    result->measurement->wait_time = other->measurement->wait_time;
-    result->measurement->object = Qnil;
+    prof_call_tree_t* result = prof_call_tree_create(other->method, other->parent, other->source_file, other->source_line);
+    result->measurement = prof_measurement_copy(other->measurement);
 
     return result;
 }
 
-void prof_call_tree_merge(prof_call_tree_t* result, prof_call_tree_t* other)
-{
-    result->measurement->called += other->measurement->called;
-    result->measurement->total_time += other->measurement->total_time;
-    result->measurement->self_time += other->measurement->self_time;
-    result->measurement->wait_time += other->measurement->wait_time;
-}
-
 static int prof_call_tree_collect_children(st_data_t key, st_data_t value, st_data_t result)
 {
     prof_call_tree_t* call_tree = (prof_call_tree_t*)value;
@@ -75,7 +56,7 @@ void prof_call_tree_mark(void* data)
     prof_call_tree_t* call_tree = (prof_call_tree_t*)data;
 
     if (call_tree->object != Qnil)
-        rb_gc_mark(call_tree->object);
+        rb_gc_mark_movable(call_tree->object);
 
     if (call_tree->source_file != Qnil)
         rb_gc_mark(call_tree->source_file);
@@ -89,13 +70,10 @@ void prof_call_tree_mark(void* data)
         rb_st_foreach(call_tree->children, prof_call_tree_mark_children, 0);
 }
 
-static void prof_call_tree_ruby_gc_free(void* data)
+void prof_call_tree_compact(void* data)
 {
-    if (data)
-    {
-        prof_call_tree_t* call_tree = (prof_call_tree_t*)data;
-        call_tree->object = Qnil;
-    }
+    prof_call_tree_t* call_tree = (prof_call_tree_t*)data;
+    call_tree->object = rb_gc_location(call_tree->object);
 }
 
 static int prof_call_tree_free_children(st_data_t key, st_data_t value, st_data_t data)
@@ -126,6 +104,27 @@ void prof_call_tree_free(prof_call_tree_t* call_tree_data)
     xfree(call_tree_data);
 }
 
+static void prof_call_tree_ruby_gc_free(void* data)
+{
+  prof_call_tree_t* call_tree = (prof_call_tree_t*)data;
+
+  if (!call_tree)
+  {
+    // Object has already been freed by C code
+    return;
+  }
+  else if (call_tree->owner == OWNER_RUBY)
+  {
+    // Ruby owns this object, we need to free the underlying C struct
+    prof_call_tree_free(call_tree);
+  }
+  else
+  {
+    // The Ruby object is being freed, but not the underlying C structure. So unlink the two.
+    call_tree->object = Qnil;
+  }
+}
+
 size_t prof_call_tree_size(const void* data)
 {
     return sizeof(prof_call_tree_t);
@@ -139,6 +138,7 @@ static const rb_data_type_t call_tree_type =
         .dmark = prof_call_tree_mark,
         .dfree = prof_call_tree_ruby_gc_free,
         .dsize = prof_call_tree_size,
+        .dcompact = prof_call_tree_compact
     },
     .data = NULL,
     .flags = RUBY_TYPED_FREE_IMMEDIATELY
@@ -156,6 +156,8 @@ VALUE prof_call_tree_wrap(prof_call_tree_t* call_tree)
 static VALUE prof_call_tree_allocate(VALUE klass)
 {
     prof_call_tree_t* call_tree = prof_call_tree_create(NULL, NULL, Qnil, 0);
+    // This object is being created by Ruby
+    call_tree->owner = OWNER_RUBY;
     call_tree->object = prof_call_tree_wrap(call_tree);
     return call_tree->object;
 }
@@ -191,19 +193,39 @@ prof_call_tree_t* call_tree_table_lookup(st_table* table, st_data_t key)
     }
 }
 
-uint32_t prof_call_figure_depth(prof_call_tree_t* call_tree_data)
+uint32_t prof_call_tree_figure_depth(prof_call_tree_t* call_tree)
 {
     uint32_t result = 0;
 
-    while (call_tree_data->parent)
+    while (call_tree->parent)
     {
         result++;
-        call_tree_data = call_tree_data->parent;
+        call_tree = call_tree->parent;
     }
 
     return result;
 }
 
+int prof_call_tree_collect_methods(st_data_t key, st_data_t value, st_data_t result)
+{
+  prof_call_tree_t* call_tree = (prof_call_tree_t*)value;
+  VALUE arr = (VALUE)result;
+  rb_ary_push(arr, prof_method_wrap(call_tree->method));
+
+  rb_st_foreach(call_tree->children, prof_call_tree_collect_methods, result);
+  return ST_CONTINUE;
+};
+
+VALUE prof_call_tree_methods(prof_call_tree_t* call_tree)
+{
+    VALUE result = rb_ary_new();
+    rb_ary_push(result, prof_method_wrap(call_tree->method));
+
+    rb_st_foreach(call_tree->children, prof_call_tree_collect_methods, result);
+
+    return result;
+}
+
 void prof_call_tree_add_parent(prof_call_tree_t* self, prof_call_tree_t* parent)
 {
     prof_call_tree_add_child(parent, self);
@@ -213,10 +235,26 @@ void prof_call_tree_add_parent(prof_call_tree_t* self, prof_call_tree_t* parent)
 void prof_call_tree_add_child(prof_call_tree_t* self, prof_call_tree_t* child)
 {
     call_tree_table_insert(self->children, child->method->key, child);
+    
+    // The child is now managed by C since its parent will free it
+    child->owner = OWNER_C;
 }
 
 /* =======  RubyProf::CallTree   ========*/
 
+/* call-seq:
+   new(method_info) -> call_tree
+
+Creates a new CallTree instance. +Klass+ should be a reference to
+a Ruby class and +method_name+ a symbol identifying one of its instance methods.*/
+static VALUE prof_call_tree_initialize(VALUE self, VALUE method_info)
+{
+  prof_call_tree_t* call_tree_ptr = prof_get_call_tree(self);
+  call_tree_ptr->method = prof_get_method(method_info);
+
+  return self;
+}
+
 /* call-seq:
    parent -> call_tree
 
@@ -242,6 +280,29 @@ static VALUE prof_call_tree_children(VALUE self)
     return result;
 }
 
+/* call-seq:
+   add_child(call_tree) -> call_tree
+
+Adds the specified call_tree as a child. If the method represented by the call tree is
+already a child than a IndexError is thrown.
+
+The returned value is the added child*/
+static VALUE prof_call_tree_add_child_ruby(VALUE self, VALUE child)
+{
+  prof_call_tree_t* parent_ptr = prof_get_call_tree(self);
+  prof_call_tree_t* child_ptr = prof_get_call_tree(child);
+
+  prof_call_tree_t* existing_ptr = call_tree_table_lookup(parent_ptr->children, child_ptr->method->key);
+  if (existing_ptr)
+  {
+    rb_raise(rb_eIndexError, "Child call tree already exists");
+  }
+
+  prof_call_tree_add_parent(child_ptr, parent_ptr);
+
+  return child;
+}
+
 /* call-seq:
    called -> MethodInfo
 
@@ -269,7 +330,7 @@ static VALUE prof_call_tree_measurement(VALUE self)
 static VALUE prof_call_tree_depth(VALUE self)
 {
     prof_call_tree_t* call_tree_data = prof_get_call_tree(self);
-    uint32_t depth = prof_call_figure_depth(call_tree_data);
+    uint32_t depth = prof_call_tree_figure_depth(call_tree_data);
     return rb_int_new(depth);
 }
 
@@ -294,12 +355,83 @@ static VALUE prof_call_tree_line(VALUE self)
     return INT2FIX(result->source_line);
 }
 
+// Helper class that lets us pass additional information to prof_call_tree_merge_children
+typedef struct self_info_t
+{
+  prof_call_tree_t* call_tree;
+  st_table* method_table;
+} self_info_t;
+
+
+static int prof_call_tree_merge_children(st_data_t key, st_data_t value, st_data_t data)
+{
+    prof_call_tree_t* other_child_ptr = (prof_call_tree_t*)value;
+
+    self_info_t* self_info = (self_info_t*)data;
+    prof_call_tree_t* self_ptr = self_info->call_tree;
+
+    prof_call_tree_t* self_child = call_tree_table_lookup(self_ptr->children, other_child_ptr->method->key);
+    if (self_child)
+    {
+        // Merge measurements
+        prof_measurement_merge_internal(self_child->measurement, other_child_ptr->measurement);
+    }
+    else
+    {
+        // Get pointer to method the other call tree invoked
+        prof_method_t* method_ptr = method_table_lookup(self_info->method_table, other_child_ptr->method->key);
+      
+        // Now copy the other call tree, reset its method pointer, and add it as a child
+        self_child = prof_call_tree_copy(other_child_ptr);
+        self_child->method = method_ptr;
+        prof_call_tree_add_child(self_ptr, self_child);
+
+        // Now tell the method that this call tree invoked it
+        prof_add_call_tree(method_ptr->call_trees, self_child);
+    }
+
+    // Recurse down a level to merge children
+    self_info_t child_info = { .call_tree = self_child, .method_table = self_info->method_table };
+    rb_st_foreach(other_child_ptr->children, prof_call_tree_merge_children, (st_data_t)&child_info);
+
+    return ST_CONTINUE;
+}
+
+void prof_call_tree_merge_internal(prof_call_tree_t* self, prof_call_tree_t* other, st_table* self_method_table)
+{
+    // Make sure the methods are the same
+    if (self->method->key != other->method->key)
+        return;
+
+    // Make sure the parents are the same.
+    // 1. They can both be set and be equal
+    // 2. They can both be unset (null)
+    if (self->parent && other->parent)
+    {
+        if (self->parent->method->key != other->parent->method->key)
+            return;
+    }
+    else if (self->parent || other->parent)
+    {
+        return;
+    }
+
+    // Merge measurements
+    prof_measurement_merge_internal(self->measurement, other->measurement);
+
+    // Now recursively descend through the call trees
+    self_info_t self_info = { .call_tree = self, .method_table = self_method_table };
+    rb_st_foreach(other->children, prof_call_tree_merge_children, (st_data_t)&self_info);
+}
+
 /* :nodoc: */
 static VALUE prof_call_tree_dump(VALUE self)
 {
     prof_call_tree_t* call_tree_data = prof_get_call_tree(self);
     VALUE result = rb_hash_new();
 
+    rb_hash_aset(result, ID2SYM(rb_intern("owner")), INT2FIX(call_tree_data->owner));
+
     rb_hash_aset(result, ID2SYM(rb_intern("measurement")), prof_measurement_wrap(call_tree_data->measurement));
 
     rb_hash_aset(result, ID2SYM(rb_intern("source_file")), call_tree_data->source_file);
@@ -320,6 +452,8 @@ static VALUE prof_call_tree_load(VALUE self, VALUE data)
     prof_call_tree_t* call_tree = prof_get_call_tree(self);
     call_tree->object = self;
 
+    call_tree->owner = FIX2INT(rb_hash_aref(data, ID2SYM(rb_intern("owner"))));
+
     VALUE measurement = rb_hash_aref(data, ID2SYM(rb_intern("measurement")));
     call_tree->measurement = prof_get_measurement(measurement);
 
@@ -346,17 +480,18 @@ static VALUE prof_call_tree_load(VALUE self, VALUE data)
     return data;
 }
 
-void rp_init_call_tree()
+void rp_init_call_tree(void)
 {
     /* CallTree */
     cRpCallTree = rb_define_class_under(mProf, "CallTree", rb_cObject);
-    rb_undef_method(CLASS_OF(cRpCallTree), "new");
     rb_define_alloc_func(cRpCallTree, prof_call_tree_allocate);
+    rb_define_method(cRpCallTree, "initialize", prof_call_tree_initialize, 1);
 
-    rb_define_method(cRpCallTree, "parent", prof_call_tree_parent, 0);
-    rb_define_method(cRpCallTree, "children", prof_call_tree_children, 0);
     rb_define_method(cRpCallTree, "target", prof_call_tree_target, 0);
     rb_define_method(cRpCallTree, "measurement", prof_call_tree_measurement, 0);
+    rb_define_method(cRpCallTree, "parent", prof_call_tree_parent, 0);
+    rb_define_method(cRpCallTree, "children", prof_call_tree_children, 0);
+    rb_define_method(cRpCallTree, "add_child", prof_call_tree_add_child_ruby, 1);
 
     rb_define_method(cRpCallTree, "depth", prof_call_tree_depth, 0);
     rb_define_method(cRpCallTree, "source_file", prof_call_tree_source_file, 0);
