@@ -1,7 +1,6 @@
 /* Copyright (C) 2005-2013 Shugo Maeda <shugo@ruby-lang.org> and Charlie Savage <cfis@savagexi.com>
    Please see the LICENSE file for copyright and distribution information */
 
-#include "rp_aggregate_call_tree.h"
 #include "rp_call_trees.h"
 #include "rp_measurement.h"
 
@@ -22,7 +21,7 @@ prof_call_trees_t* prof_get_call_trees(VALUE self)
     return result;
 }
 
-prof_call_trees_t* prof_call_trees_create()
+prof_call_trees_t* prof_call_trees_create(void)
 {
     prof_call_trees_t* result = ALLOC(prof_call_trees_t);
     result->start = ALLOC_N(prof_call_tree_t*, INITIAL_CALL_TREES_SIZE);
@@ -68,11 +67,11 @@ void prof_call_trees_ruby_gc_free(void* data)
     }
 }
 
-static int prof_call_trees_collect_aggregates(st_data_t key, st_data_t value, st_data_t data)
+static int prof_call_trees_collect(st_data_t key, st_data_t value, st_data_t data)
 {
     VALUE result = (VALUE)data;
     prof_call_tree_t* call_tree_data = (prof_call_tree_t*)value;
-    VALUE aggregate_call_tree = prof_aggregate_call_tree_wrap(call_tree_data);
+    VALUE aggregate_call_tree = prof_call_tree_wrap(call_tree_data);
     rb_ary_push(result, aggregate_call_tree);
 
     return ST_CONTINUE;
@@ -87,11 +86,16 @@ static int prof_call_trees_collect_callees(st_data_t key, st_data_t value, st_da
 
     if (rb_st_lookup(callers, call_tree_data->method->key, (st_data_t*)&aggregate_call_tree_data))
     {
-        prof_call_tree_merge(aggregate_call_tree_data, call_tree_data);
+      prof_measurement_merge_internal(aggregate_call_tree_data->measurement, call_tree_data->measurement);
     }
     else
     {
+        // Copy the call tree so we don't touch the original and give Ruby ownerhip 
+        // of it so that it is freed on GC
         aggregate_call_tree_data = prof_call_tree_copy(call_tree_data);
+        aggregate_call_tree_data->owner = OWNER_RUBY;
+
+
         rb_st_insert(callers, call_tree_data->method->key, (st_data_t)aggregate_call_tree_data);
     }
 
@@ -166,7 +170,7 @@ VALUE prof_call_trees_min_depth(VALUE self)
     prof_call_trees_t* call_trees = prof_get_call_trees(self);
     for (prof_call_tree_t** p_call_tree = call_trees->start; p_call_tree < call_trees->ptr; p_call_tree++)
     {
-        unsigned int call_tree_depth = prof_call_figure_depth(*p_call_tree);
+        unsigned int call_tree_depth = prof_call_tree_figure_depth(*p_call_tree);
         if (call_tree_depth < depth)
             depth = call_tree_depth;
     }
@@ -210,17 +214,21 @@ VALUE prof_call_trees_callers(VALUE self)
 
         if (rb_st_lookup(callers, parent->method->key, (st_data_t*)&aggregate_call_tree_data))
         {
-            prof_call_tree_merge(aggregate_call_tree_data, *p_call_tree);
+          prof_measurement_merge_internal(aggregate_call_tree_data->measurement, (*p_call_tree)->measurement);
         }
         else
         {
+            // Copy the call tree so we don't touch the original and give Ruby ownerhip 
+            // of it so that it is freed on GC
             aggregate_call_tree_data = prof_call_tree_copy(*p_call_tree);
+            aggregate_call_tree_data->owner = OWNER_RUBY;
+
             rb_st_insert(callers, parent->method->key, (st_data_t)aggregate_call_tree_data);
         }
     }
 
     VALUE result = rb_ary_new_capa((long)callers->num_entries);
-    rb_st_foreach(callers, prof_call_trees_collect_aggregates, result);
+    rb_st_foreach(callers, prof_call_trees_collect, result);
     rb_st_free_table(callers);
     return result;
 }
@@ -240,7 +248,7 @@ VALUE prof_call_trees_callees(VALUE self)
     }
 
     VALUE result = rb_ary_new_capa((long)callees->num_entries);
-    rb_st_foreach(callees, prof_call_trees_collect_aggregates, result);
+    rb_st_foreach(callees, prof_call_trees_collect, result);
     rb_st_free_table(callees);
     return result;
 }
@@ -271,7 +279,7 @@ VALUE prof_call_trees_load(VALUE self, VALUE data)
     return data;
 }
 
-void rp_init_call_trees()
+void rp_init_call_trees(void)
 {
     cRpCallTrees = rb_define_class_under(mProf, "CallTrees", rb_cObject);
     rb_undef_method(CLASS_OF(cRpCallTrees), "new");
